package com.muyu.cloud.vehicle.gateway.domain.model;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.log4j.Log4j2;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @ Description：任务执行模型
 */
@Data
@Log4j2
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class TaskModel {
    /**
     * 任务状态 默认为false状态
     * true为执行中，false为未执行
     */
    private final AtomicBoolean status =new AtomicBoolean(Boolean.FALSE);
    /**
     * 堵塞计数器
     */
    private CountDownLatch countDownLatch;
    /**
     * 任务执行堵塞队列
     */
    private LinkedBlockingDeque<String> carQueue =new LinkedBlockingDeque<>();
    /**
     * 任务是否执行
     * true 执行中
     * false 未执行
     * @return 是否有任务执行
     */
    private boolean isExecution(){
        return !status.get();
    }
    /**
     * 任务名称
     */
    private String taskName;
    /**
     * 任务执行次数
     */
    private Integer taskExecutionCount=0;
    /**
     * 任务开始时间
     */
    private Long taskStartTime;
    /**
     * 任务成功执行次数
     */
    private AtomicInteger taskSuccessSum=new AtomicInteger();
    /**
     * 任务执行失败次数
     */
    private AtomicInteger taskErrorSum=new AtomicInteger();

    /**
     * 判断是否有任务
     * @return true 有任务
     */
    public boolean hashNext(){
        return !carQueue.isEmpty();
    }

    /**
     * 获取下一个任务节点
     * @return 任务VIN
     */
    public String next(){
        return carQueue.poll();
    }

}
